'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  var defaultIdentifier = t.identifier('default');
  var rewireIdentifier = t.identifier('rewire');
  var restoreIdentifier = t.identifier('restore');
  var stubIdentifier = t.identifier('$stub');
  var VISITED = Symbol();

  var buildStub = (0, _babelTemplate2.default)('\n    export function REWIRE(STUB) {\n      LOCAL = STUB;\n    }\n  ', { sourceType: 'module' });

  var buildRestore = (0, _babelTemplate2.default)('\n    export function RESTORE() {\n      BODY\n    }\n  ', { sourceType: 'module' });

  function buildNamedExport(local, exported) {
    return markVisited(t.exportNamedDeclaration(null, [t.exportSpecifier(t.identifier(local.name), t.identifier(exported.name))]));
  }

  function markVisited(node) {
    node[VISITED] = true;
    return node;
  }

  return {
    visitor: {
      Program: {
        enter: function enter(path, state) {
          state.exports = [];
          state.hoisted = [];
        },
        exit: function exit(path, _ref2) {
          var exports = _ref2.exports,
              hoisted = _ref2.hoisted;

          if (!exports.length) return;
          // add hoisted variables to the top
          if (hoisted.length) {
            path.unshiftContainer('body', [t.variableDeclaration('var', hoisted)]);
          }

          // generate temp variables to capture original values
          var tempVars = [];
          exports.filter(function (e) {
            return !e.original;
          }).forEach(function (e) {
            var temp = e.original = path.scope.generateUidIdentifierBasedOnNode(e.exported);
            tempVars.push(t.variableDeclarator(temp, e.local));
          });

          // generate new IDs to keep sourcemaps clean
          var rewired = exports.map(function (_ref3) {
            var exported = _ref3.exported,
                local = _ref3.local,
                original = _ref3.original;
            return {
              exported: t.identifier(exported.name),
              local: t.identifier(local.name),
              original: t.identifier(original.name)
            };
          });

          // generate stub functions
          var hasConflictingBinding = path.scope.hasOwnBinding('rewire');
          var stubs = rewired.map(function (_ref4) {
            var exported = _ref4.exported,
                local = _ref4.local;

            var rewire = t.isIdentifier(exported, defaultIdentifier) && !hasConflictingBinding ? rewireIdentifier : t.identifier('rewire$' + exported.name);
            return markVisited(buildStub({
              REWIRE: rewire,
              LOCAL: local,
              STUB: stubIdentifier
            }));
          });

          // generate restore function
          var restore = path.scope.hasOwnBinding('restore') ? t.identifier('restore$rewire') : restoreIdentifier;
          var assignments = rewired.map(function (_ref5) {
            var local = _ref5.local,
                original = _ref5.original;
            return t.expressionStatement(t.assignmentExpression('=', local, original));
          });

          var body = [].concat(_toConsumableArray(stubs), [markVisited(buildRestore({ RESTORE: restore, BODY: assignments }))]);

          if (tempVars.length) {
            body.unshift(t.variableDeclaration('var', tempVars));
          }

          path.pushContainer('body', body);
        }
      },
      // export default
      ExportDefaultDeclaration: function ExportDefaultDeclaration(path, _ref6) {
        var exports = _ref6.exports,
            hoisted = _ref6.hoisted;

        var declaration = path.node.declaration;
        if (t.isIdentifier(declaration)) {
          // export default foo
          var binding = path.scope.getBinding(declaration.name);
          if (binding.kind === 'const') return; // ignore constants
          exports.push({ exported: defaultIdentifier, local: declaration });
          path.replaceWith(buildNamedExport(declaration, defaultIdentifier));
        } else if (t.isFunctionDeclaration(declaration)) {
          //export default function () {}
          var id = declaration.id || path.scope.generateUidIdentifier('default');
          declaration.id = path.scope.generateUidIdentifierBasedOnNode(declaration.id);
          exports.push({ exported: defaultIdentifier, local: id, original: declaration.id });
          path.replaceWithMultiple([declaration, buildNamedExport(id, defaultIdentifier)]);
          hoisted.push(t.variableDeclarator(id, declaration.id));
        } else if (t.isClassDeclaration(declaration)) {
          //export default class {}
          var _id = declaration.id || path.scope.generateUidIdentifier('default');
          exports.push({ exported: defaultIdentifier, local: _id });
          path.replaceWithMultiple([t.variableDeclaration('var', [t.variableDeclarator(_id, t.classExpression(declaration.id, declaration.superClass, declaration.body, declaration.decorators || []))]), buildNamedExport(_id, defaultIdentifier)]);
        } else {
          // export default ...
          var _id2 = path.scope.generateUidIdentifier('default');
          exports.push({ exported: defaultIdentifier, local: _id2 });
          path.replaceWithMultiple([t.variableDeclaration('var', [t.variableDeclarator(_id2, declaration)]), buildNamedExport(_id2, defaultIdentifier)]);
        }
      },
      // export {}
      ExportNamedDeclaration: function ExportNamedDeclaration(path, _ref7) {
        var exports = _ref7.exports,
            hoisted = _ref7.hoisted;

        if (path.node[VISITED]) return;
        // export { foo } from './bar.js'
        if (path.node.source) return;

        var declaration = path.node.declaration;
        if (t.isVariableDeclaration(declaration)) {
          // export const foo = 'bar'
          if (declaration.kind === 'const') return;
          // export var foo
          declaration.declarations.forEach(function (_ref8) {
            var id = _ref8.id;

            exports.push({ exported: id, local: id });
          });
        } else if (t.isFunctionDeclaration(declaration)) {
          // export function foo() {}
          var id = declaration.id;
          declaration.id = path.scope.generateUidIdentifierBasedOnNode(id);
          exports.push({ exported: id, local: id, original: declaration.id });
          path.replaceWithMultiple([declaration, buildNamedExport(id, id)]);
          hoisted.push(t.variableDeclarator(id, declaration.id));
        } else if (t.isClassDeclaration(declaration)) {
          // export function class foo {}
          var _id3 = declaration.id;
          exports.push({ exported: _id3, local: _id3 });
          path.replaceWithMultiple([t.variableDeclaration('var', [t.variableDeclarator(_id3, t.classExpression(_id3, declaration.superClass, declaration.body, declaration.decorators || []))]), buildNamedExport(_id3, _id3)]);
        } else {
          // export {foo}
          path.node.specifiers.forEach(function (_ref9) {
            var exported = _ref9.exported,
                local = _ref9.local;

            var binding = path.scope.getBinding(local.name);
            if (binding.kind === 'const') return; // ignore constants
            exports.push({ exported: exported, local: local });
          });
        }
      }
    }
  };
};

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }